
import os
from typing import Tuple

from tensorflow.keras import layers, Model, regularizers

script_directory = os.path.dirname(os.path.abspath(__file__))

def get_model_path(state='on_entire_data'):

    if state == 'on_entire_data':

        print('This model was trained on entire (Train + Test) data! If you want to use model that was trained on only Train samples please use load_weights(model_state=\'on_train_data\')')
        return os.path.join(script_directory, "pretrained", "sumonet3.h5")

    elif state == 'on_train_data':

        print('This model was trained on the Train data! If you want to use final model please use load_weights(model_state=\'on_entire_data\') ')
        return os.path.join(script_directory, "pretrained", "sumonet3_partial.h5")

    else:
        
        raise ValueError('model_state just takes \'on_entire_data\' or \'on_train_data\' parameters')


class SUMOnet(Model):

    def __init__(self,input_shape:Tuple[int,int]=(21, 24)) -> None:
        
        super().__init__()
        
        self.cnn = layers.Conv1D(128,2,padding='valid',activation='relu',kernel_initializer='he_normal',strides=1)
        self.bigru = layers.Bidirectional(layers.GRU(16, dropout=0.4, recurrent_dropout=0,return_sequences=True))
        self.pool = layers.GlobalAveragePooling1D()
        self.dense64 = layers.Dense(64,kernel_initializer='he_normal',activity_regularizer= regularizers.l2(1e-4))
        self.dropout = layers.Dropout(0.4)
        self.relu = layers.Activation('relu')
        self.dense128_1 = layers.Dense(128,kernel_initializer='he_normal',activity_regularizer= regularizers.l2(1e-4))
        self.dropout_1 = layers.Dropout(0.4)
        self.relu_1 = layers.Activation('relu')
        self.dense128_2 = layers.Dense(128,kernel_initializer='he_normal',activity_regularizer= regularizers.l2(1e-4))
        self.dropout_2 = layers.Dropout(0.4)
        self.relu_2 = layers.Activation('relu')
        self.dense2 = layers.Dense(2, kernel_initializer='he_normal')
        self.softmax = layers.Activation('softmax')

        self.build((None, input_shape[0], input_shape[1])) #input shape is batch,21,24 for blosum62 encoded data

    def call(self, inputs):

        x = self.cnn(inputs)
        x = self.bigru(x)
        x = self.pool(x)
        x = self.dense64(x)
        x = self.dropout(x)
        x = self.relu(x)
        x = self.dense128_1(x)
        x = self.dropout_1(x)
        x = self.relu_1(x)
        x = self.dense128_2(x)
        x = self.dropout_2(x)
        x = self.relu_2(x)
        x = self.dense2(x)
        x = self.softmax(x)
        
        return x

    def load_weights(self,model_state='on_entire_data'):

        preTrainedModelPath = get_model_path(model_state)
        super().load_weights(preTrainedModelPath)





from sklearn import metrics

def f1_score(y_true, y_probs):

    return metrics.f1_score(y_true, y_probs.argmax(-1))

def mcc(y_true, y_probs):

    return metrics.matthews_corrcoef(y_true, y_probs.argmax(-1))

def rocAuc_score(y_true, y_probs):

    lr_fpr, lr_tpr, _ = metrics.roc_curve(y_true, y_probs[:,1])

    return metrics.auc(lr_fpr, lr_tpr)

def aupr_score(y_true, y_probs):

    precision, recall, _ = metrics.precision_recall_curve(y_true, y_probs[:,1])
    
    return metrics.auc(recall, precision)

def evaluate(y_true, y_probs, metrics_):
    
    """
    Args:

        y_true: gold labels -> It is a 2-dim array: [[1,0],[0,1],[1,0],[1,0]]
        y_probs: predicted probabilities -> It is a 2-dim array: [[0.4,0.6],[0.2,0.8],[0.7,0.3],[0.6,0.4]]
        metrics: evaluation metrics -> can be a list of strings ['f1', 'mcc', 'roc', 'aupr'] or a string 'roc'

    Output:

        outputEval: a dictionary that contains evaluation metric name and its value -> {'f1':0.72, 'mcc': 0.42}
    """


    metricsDict = {'f1':f1_score, 'mcc':mcc, 'roc':rocAuc_score, 'aupr':aupr_score}
    outputEval = {}

    if type(metrics_) == str:

        if metrics_.lower() not in metricsDict:

                raise ValueError(f"{metrics_.lower()} is not supported! f1, mcc, roc and aupr are supported evaluation metrics")

        else:

            outputEval[metrics_.lower()]  = metricsDict[metrics_.lower()](y_true, y_probs)

        return outputEval

    elif type(metrics_) == list:

        metricNames = [x.lower() for x in metrics_]

        for metricName in metricNames:

            if metricName not in metricsDict:

                raise ValueError(f"{metricName} is not supported! f1, mcc, roc and aupr are supported evaluation metrics")

            else:

                outputEval[metricName]  = metricsDict[metricName](y_true, y_probs)

        return outputEval

    else:

        raise TypeError("metrics must be a string or list of strings")




import os
import joblib
import numpy as np
import pandas as pd
import epitopepredict as ep

from sumonet.utils.data_pipe import Data
from typing import List

script_directory = os.path.dirname(os.path.abspath(__file__))

def get_min_max_scaler_path():

        return os.path.join(script_directory, "scaler", "minmax_scaler.gz")



def create_dict():
    # Global function that is used for one-hot encoding
    # Function works like a map

    letterDict = {}
    letterDict["A"] = 0
    letterDict["C"] = 1
    letterDict["D"] = 2
    letterDict["E"] = 3
    letterDict["F"] = 4
    letterDict["G"] = 5
    letterDict["H"] = 6
    letterDict["I"] = 7
    letterDict["K"] = 8
    letterDict["L"] = 9
    letterDict["M"] = 10
    letterDict["N"] = 11
    letterDict["P"] = 12
    letterDict["Q"] = 13
    letterDict["R"] = 14
    letterDict["S"] = 15
    letterDict["T"] = 16
    letterDict["V"] = 17
    letterDict["W"] = 18
    letterDict["Y"] = 19
    letterDict["X"] = 20 
    return letterDict

class Encoding():

    def __init__(self, encoderType: str='blosum62' ,scaler: bool=True) -> None:

        self.encoderType = encoderType
        self.scaler = scaler


        self.encodings = create_dict() # Dictionary for one-hot encoding
        self.blosum = ep.blosum62
        self.nlf = pd.read_csv('https://raw.githubusercontent.com/dmnfarrell/epitopepredict/master/epitopepredict/mhcdata/NLF.csv',index_col=0)

        self.sequences = []
        self.X = None

    def set_encoder_type(self,encoderType):
        self.encoderType = encoderType

    def one_hot(self,data):

        oneHot_data = np.zeros((len(data),len(data[0]),len(self.encodings)))

        for sample_index, sequence in enumerate(data):
            
            for feature_index, aa in enumerate(sequence):

                encode = self.encodings[aa]
                oneHot_data[sample_index][feature_index][encode] = 1
                
        self.X = oneHot_data     

    def blosum_helper(self,seq):
        #encode a peptide into blosum features

        x = pd.DataFrame([self.blosum[i] for i in seq]).reset_index(drop=True)

        e = x.values.flatten()    

        return e

    def bl_encoder(self,data):
        
        bl_data = list(map(self.blosum_helper,data))
            
        bl_data = np.asarray(bl_data,dtype='float32')

        self.X = bl_data        

    def nlf_helper(self,seq):    

        x = pd.DataFrame([self.nlf[i] for i in seq]).reset_index(drop=True) 

        e = x.values.flatten()

        return e
        
    def nlf_encoder(self,data):
                
        nlf_data = list(map(self.nlf_helper,data))

        nlf_data = np.asarray(nlf_data,dtype='float32')

        self.X = nlf_data     

    def encode(self):

        if self.encoderType.lower() == 'blosum62':
            
            self.bl_encoder(self.sequences)

        elif self.encoderType.lower() == 'nlf':
            
            self.nlf_encoder(self.sequences)

        elif self.encoderType.lower() == 'one-hot':
            
            self.one_hot(self.sequences)

        else:
            raise ValueError('EncoderType is {encoderType}. It should be blosum62, nlf or one-hot.')
    
    def minmax(self):

        minmax_scaler = joblib.load(get_min_max_scaler_path())
        self.X = minmax_scaler.transform(self.X)

    def reshape(self):

        self.X = self.X.reshape(len(self.X),21,self.X.shape[1]//21)

    def preprocess(self):
        
        if self.encoderType.lower() == 'blosum62' or self.encoderType.lower() == 'nlf':
            
            if self.scaler:
                self.minmax()

            self.reshape()


    def encode_data(self,X:List[str]):

            """
            This function is used to get encoded representation of given vectors.

            Args:

                X: List that contains samples.

            Output:

                self.X: Encoded samples
            """

            self.sequences = X

            self.encode()

            self.preprocess()

            return self.X

    


import os

from typing import List, Tuple
from os.path import dirname, abspath


parent_dir = dirname(dirname(abspath(__file__))) 



class Data:

        def __init__(self):
               
                self.posTrainDataPath, self.negTrainDataPath = self.get_train_data_path()
                self.posTestDataPath, self.negTestDataPath = self.get_test_data_path()


        @staticmethod
        def get_train_data_path():
                """
                Function that returns train data path.
        
                Returns:
                posTrainDataPath: The data path that contains positive training samples.
                negTrainDataPath: The data path that contains negative training samples.
                
                """

                posTrainDataPath =  os.path.join(parent_dir,"data","train","Sumoylation_pos_Train.fasta")
                negTrainDataPath = os.path.join(parent_dir,"data","train","Sumoylation_neg_Train.fasta") 


                return posTrainDataPath, negTrainDataPath

        @staticmethod
        def get_test_data_path():
                """
                Function that returns test data path.
        
                Returns:
                posTestDataPath: The data path that contains positive test samples.
                negTestDataPath: The data path that contains negative test samples.
                
                """

                posTestDataPath = os.path.join(parent_dir,"data","test","Sumoylation_pos_Test.fasta") 
                negTestDataPath = os.path.join(parent_dir,"data","test","Sumoylation_neg_Test.fasta")  


                return posTestDataPath, negTestDataPath

        @staticmethod
        def get_new_data(filePath:str) -> List[str]:
        
                with open(filePath) as f:
                        
                        my_array = f.read().splitlines()
        
                return my_array[1::2]

        def load_data(self) -> Tuple[List[str], List[int], List[str], List[int]]:
                """
                Function that returns our Train, Test sequences and their labels.
        
                Returns:
                X_train: Training sequences
                y_train: Training labels
                X_test: Test sequences
                y_test: Test labels
                """
                posTrainData = self.get_new_data(self.posTrainDataPath)
                negTrainData = self.get_new_data(self.negTrainDataPath)

                posTestData = self.get_new_data(self.posTestDataPath)
                negTestData = self.get_new_data(self.negTestDataPath)

                X_train = posTrainData + negTrainData
                X_test = posTestData + negTestData

                y_train = [1]*len(posTrainData) + [0]*len(negTrainData)
                y_test =  [1]*len(posTestData) + [0]*len(negTestData)


                return X_train, y_train, X_test, y_test



from setuptools import setup, find_packages


setup(
    name='sumonet',
    version='0.0.10',
    license='Academic Free License ("AFL") v. 3.0',
    author="Berke Dilekoglu",
    author_email='dilekogluberke@gmail.com',
    packages=find_packages(),
    package_data={
    # If any package or subpackage contains *.txt or *.rst files, include
    # them:
    "": ["*.fasta", "*.gz"]
    
  },
    url='https://github.com/berkedilekoglu/SUMOnet',
    download_url = 'https://github.com/berkedilekoglu/SUMOnet/archive/refs/tags/v0.0.10tar.gz', 
    keywords='sumoylation machine-learning deep-learning',
    install_requires=[
        'numpy',
        'scikit-learn',
        'joblib',
        'pandas',
        'epitopepredict',
        'tensorflow',
        'keras',
      ],

)